@@ -120,11 +120,11 @@ typedef struct tcpip_state
     int written;  /* -1 if we aren't writing */
     int towrite;  /* to verify against user input */
     int (*complete)(const char *buf, int len); /* length/complete. */
+    char *bind_host;
 #if HAVE_GETADDRINFO
     struct addrinfo *ai;
     struct addrinfo *ai_connect;
     int ipv6_only;
-    char *bind_host;
 #if RESOLVER_THREAD
     int pipefd[2];
     char *hoststr;
@@ -175,11 +175,10 @@ static struct tcpip_state *tcpip_state_create(void)
     sp->altsize = sp->altlen = 0;
     sp->towrite = sp->written = -1;
     sp->complete = cs_complete_auto;
-
+    sp->bind_host = 0;
 #if HAVE_GETADDRINFO
     sp->ai = 0;
     sp->ai_connect = 0;
-    sp->bind_host = 0;
 #if RESOLVER_THREAD
     sp->hoststr = 0;
     sp->pipefd[0] = sp->pipefd[1] = -1;
@@ -432,9 +431,9 @@ struct addrinfo *tcpip_getaddrinfo(const char *str, const char *port,
     return res;
 }
 
-#endif
+#else
 /* gethostbyname .. old systems */
-int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
+static int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
                        int default_port)
 {
     struct hostent *hp;
@@ -474,6 +473,7 @@ int tcpip_strtoaddr_ex(const char *str, struct sockaddr_in *add,
         return 0;
     return 1;
 }
+#endif
 
 #if HAVE_GETADDRINFO
 static struct addrinfo *create_net_socket(COMSTACK h)
@@ -974,11 +974,7 @@ int tcpip_listen(COMSTACK h, char *raddr, int *addrlen,
             h->cerrno = CSNODATA;
         else
         {
-#ifdef WIN32
-            shutdown(h->iofile, SD_RECEIVE);
-#else
-            shutdown(h->iofile, SHUT_RD);
-#endif
+            shutdown(h->iofile, 0); /* SHUT_RD/SHUT_RECEIVE */
             listen(h->iofile, SOMAXCONN);
             h->cerrno = CSYSERR;
         }
@@ -1368,8 +1364,8 @@ void tcpip_close(COMSTACK h)
     tcpip_state *sp = (struct tcpip_state *)h->cprivate;
 
     TRC(fprintf(stderr, "tcpip_close: h=%p pid=%d\n", h, getpid()));
-#if HAVE_GETADDRINFO
     xfree(sp->bind_host);
+#if HAVE_GETADDRINFO
 #if RESOLVER_THREAD
     if (sp->pipefd[0] != -1)
     {
@@ -1695,6 +1691,21 @@ int cs_get_peer_certificate_x509(COMSTACK cs, char **buf, int *len)
     return 0;
 }
 
+int cs_set_head_only(COMSTACK cs, int head_only)
+{
+    if (cs->type == tcpip_type || cs->type == ssl_type)
+    {
+        tcpip_state *sp = (tcpip_state *)cs->cprivate;
+        if (head_only)
+            sp->complete = cs_complete_auto_head;
+        else
+            sp->complete = cs_complete_auto;
+        return 0;
+    }
+    cs->cerrno = CS_ST_INCON;
+    return -1;
+}
+
 /*
  * Local variables:
  * c-basic-offset: 4
